/*
 * Factory-related data type macros.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>



EXTERN_C_BEGIN

/* Type and value constructor macros. */

#define ipc_varg_typedef(value_type, type) \
typedef struct { l4_umword_t tag; value_type value; } type;

#define ipc_varg_mword(type, value) \
((type) {L4_VARG_TYPE_MWORD | (sizeof(l4_mword_t) << 16), value})

#define ipc_varg_umword(type, value) \
((type) {L4_VARG_TYPE_UMWORD | (sizeof(l4_umword_t) << 16), value})

/* Tag inspection macros. */

#define ipc_varg_size(tag) ((tag >> 16) & 0xff)

#define ipc_varg_type(tag) (tag & 0xff)

#define ipc_varg_word(tag) \
((ipc_varg_type(tag) == L4_VARG_TYPE_MWORD) || \
 (ipc_varg_type(tag) == L4_VARG_TYPE_UMWORD))

/* Convenience macro for creating a type name, not necessarily employed by all
   types. */

#define ipc_varg(type) ipc_varg_##type##_t

EXTERN_C_END
